package com.fxcm.breakout.strategy.fc.backtester;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import com.fxcm.breakout.strategy.fc.priceRetriever.*;
import com.fxcm.breakout.strategy.fc.strategyBuilder.*;
import com.fxcore2.*;

public class BacktestTester {

	static List<Candlestick> candleSticksList;

	public static void main(String[] args) {
		O2GSession session = null;
		double stopLoss = 0.003;
		double takeProfit = 0.003;
		int bottomRSI = 10;
		List<StrategyResult> strategySummary = new ArrayList<StrategyResult>();
		
		try {
			LoginParams loginParams = new LoginParams("d101592791", "d101592791",
					"http://www.fxcorporate.com/Hosts.jsp", "Demo");
			SampleParams sampleParams = new SampleParams("EUR/USD", "m1", "01.11.2017 00:00", "01.12.2017 00:00");
			// if you supply parameters from the command line
			/*
			 * LoginParams loginParams = new LoginParams(args); SampleParams
			 * sampleParams = new SampleParams(args);
			 */
			// Tester class continued...
			session = O2GTransport.createSession();
			SessionStatusListener statusListener = new SessionStatusListener();
			session.subscribeSessionStatus(statusListener);
			session.login(loginParams.getLogin(), loginParams.getPassword(), loginParams.getUrl(),
					loginParams.getConnection());
			if (statusListener.waitEvents() && statusListener.isConnected()) {
				// connected...
				ResponseListener responseListener = new ResponseListener();
				session.subscribeResponse(responseListener);
				HistoryMiner.getHistoricalPrices(session, sampleParams, responseListener);
				session.logout();
				statusListener.waitEvents();
				session.unsubscribeResponse(responseListener);
			} else {
				System.out.println("Timeout expired before connection could be established");
			}
			session.unsubscribeSessionStatus(statusListener);
			if (HistoryMiner.candlesticksList != null) {
				candleSticksList = HistoryMiner.candlesticksList;
				HistoryMiner.printHistoricalPrices(candleSticksList);
				BreakoutStrategy.calculateDonchianIndicatorForDataSet(candleSticksList, 14);
				BreakoutStrategy breakage = new BreakoutStrategy();
				double CEILING_DONCHIAN = breakage.getUpperDonchianChannel(candleSticksList);
				breakage.getLowerDonchianChannel(candleSticksList);
				double FLOOR_DONCHIAN = breakage.getUpperDonchianChannel(candleSticksList);
				
				for (double i = stopLoss; i < stopLoss+0.005 ; i+=0.0005) { 
					for (double j = takeProfit; j < takeProfit+0.005; j+=0.0005) { 
							Strategy breakoutStrategy = new BreakoutStrategy(i, j, FLOOR_DONCHIAN, CEILING_DONCHIAN);
							StrategyResult sr = ((BreakoutStrategy) breakoutStrategy).runStrategy(candleSticksList);
							strategySummary.add(sr);
					}
				}
				Collections.sort(strategySummary, new StrategyResult()); // sort
																			// results
																			// list
				double avergeProfit = StrategyResult.calculateAvgStrategyProfit(strategySummary);
				System.out.println("Average profit=> " + (double) Math.round(10000 * avergeProfit) / 100 + "%");
				System.out.println("10 best results ______________________________________________________________ ");
				for (int i = 0; i < 10; i++) {
					BreakoutStrategy breakout = (BreakoutStrategy) strategySummary.get(i).getStrategy();
					System.out.println("profit:" + (double) Math.round(strategySummary.get(i).getProfit() * 10000) / 100
							+ "%" + " | max-drawdown:"
							+ (double) Math.round(strategySummary.get(i).getMaxDrawdown() * 10000) / 100 + "%" + " | wins:"
							+ (double) Math.round(10000 * strategySummary.get(i).getWinsRatio()) / 100 + "%" + " | losses:"
							+ (double) Math.round(10000 * strategySummary.get(i).getLossesRatio()) / 100 + "%" + " | s-l:"
							+ (double) Math.round(breakout.getSTOP_LOSS() * 10000) / 100 + "%" + " | t-p:"
							+ (double) Math.round(breakout.getTAKE_PROFIT() * 10000) / 100 + "%" + " | bottom-rsi:"
							+ breakout.getFLOOR_DONCHIAN() + " | top-rsi:" + breakout.getCEILING_DONCHIAN() + "\n");
				}
				System.out.println("10 worst results _____________________________________________________________ ");
				for (int i = strategySummary.size() - 1; i > strategySummary.size() - 11; i--) {
					BreakoutStrategy breakout = (BreakoutStrategy) strategySummary.get(i).getStrategy();
					System.out.println("profit: " + (double) Math.round(strategySummary.get(i).getProfit() * 10000) / 100
							+ "%" + " | max-drawdown:"
							+ (double) Math.round(strategySummary.get(i).getMaxDrawdown() * 10000) / 100 + "%" + " | wins:"
							+ (double) Math.round(10000 * strategySummary.get(i).getWinsRatio()) / 100 + "%" + " | losses:"
							+ (double) Math.round(10000 * strategySummary.get(i).getLossesRatio()) / 100 + "%" + " | s-l:"
							+ (double) Math.round(breakout.getSTOP_LOSS() * 10000) / 100 + "%" + " | t-p:"
							+ (double) Math.round(breakout.getTAKE_PROFIT() * 10000) / 100 + "%" + " | bottom-rsi:"
							+ breakout.getFLOOR_DONCHIAN() + " | top-rsi:" + breakout.getCEILING_DONCHIAN() + "\n");
				}
				
			}
		} catch (Exception e) {
			System.out.println("Exception: " + e.toString());
		} finally {
			if (session != null)
				session.dispose();
		}
	}

}
